#!/usr/bin/env python3
"""
Module PYSIDE2ACTION -- Python PySide2 Action Objects
Sub-Package GUI.TOOLKITS.PYSIDE2 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for handling user actions.
"""

from PySide2 import QtCore as qtc, QtGui as qtg, QtWidgets as qt

from plib.gui._base import action
from plib.gui.defs import *

from ._pyside2common import _PQtCommunicator, _PQtWidgetBase


class PQtMenu(_PQtWidgetBase, qt.QMenuBar, action.PMenuBase):
    """Customized Qt menu class.
    """
    
    popupclass = qt.QMenu
    
    def __init__(self, mainwidget):
        qt.QMenuBar.__init__(self, mainwidget)
        action.PMenuBase.__init__(self, mainwidget)
        mainwidget.setMenuBar(self)
    
    def _add_popup(self, title, popup):
        popup.setTitle(title)
        self.addMenu(popup)
    
    def _add_popup_action(self, act, popup):
        popup.addAction(act)


class PQtToolBar(_PQtWidgetBase, qt.QToolBar, action.PToolBarBase):
    """Customized Qt toolbar class.
    """
    
    def __init__(self, mainwidget):
        qt.QToolBar.__init__(self, mainwidget)
        if mainwidget.large_icons:
            sz = self.iconSize()
            self.setIconSize(qtc.QSize(
                *[i * 3 / 2 for i in (sz.width(), sz.height())]))
        if mainwidget.show_labels:
            style = qtc.Qt.ToolButtonTextUnderIcon
        else:
            style = qtc.Qt.ToolButtonIconOnly
        self.setToolButtonStyle(style)
        action.PToolBarBase.__init__(self, mainwidget)
        mainwidget.addToolBar(self)
    
    def add_action(self, act):
        self.addAction(act)
    
    def add_separator(self):
        self.addSeparator()


class PQtAction(_PQtCommunicator, qt.QAction, action.PActionBase):
    """Customized Qt action class.
    """
    
    def __init__(self, key, mainwidget):
        qt.QAction.__init__(self, mainwidget)
        self.setIcon(qtg.QIcon(qtg.QPixmap(self.get_icon_filename(key))))
        self.setText(self.get_menu_str(key))
        self.setToolTip(self.get_toolbar_str(key))
        s = self.get_accel_str(key)
        if s is not None:
            self.setShortcut(qtg.QKeySequence(s))
        action.PActionBase.__init__(self, key, mainwidget)
    
    def enable(self, enabled):
        self.setEnabled(enabled)



#!/usr/bin/env python3
"""
Module PANELS -- Dialog Classes for GUI
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines panel classes for use with PLIB3.GUI applications.
The following classes are included:

- PMainPanel: adds convenience items to PAutoPanel to
  make it easier to use as the client widget of your GUI
  application's main window.
"""

from plib.stdlib.decotools import cached_property

from plib.gui import main as gui
from plib.gui import specs
from plib.gui.defs import *


class PMainPanel(gui.PAutoPanel):
    """Panel widget specialized to be the client of an app's main window.
    
    This widget will also work in any situation where it is a top-level
    container (e.g., the main panel of a tab in a tab widget).
    """
    
    baseclass = gui.PAutoPanel  # so sub-panels will construct properly
    
    align = ALIGN_JUST
    layout = LAYOUT_NONE
    maintitle = None
    margin = specs.framemargin
    spacing = specs.panelspacing
    style = PANEL_NONE
    
    def __init__(self, parent):
        align = self.align
        layout = self.layout
        margin = self.margin
        spacing = self.spacing
        style = self.style
        gui.PAutoPanel.__init__(self, parent, layout=layout, style=style,
                                align=align, margin=margin, spacing=spacing)
        
        # Set parent's caption from our maintitle attribute, if defined
        if self.maintitle:
            parent.set_caption(self.maintitle)
    
    @cached_property
    def messagebox(self):
        return self._parent.messagebox



#!/usr/bin/env python3
"""
Module WXAPP -- Python wxWidgets Application Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI application objects.
"""

import wx

from plib.gui.defs import *
from plib.gui._base import app

from ._wxcommon import _PWxCommunicator, _wxicons


class PWxMessageBox(app.PMessageBoxBase):
    """Customized wxWidgets message box.
    """
    
    questionmap = {
        answerYes: wx.ID_YES,
        answerNo: wx.ID_NO,
        answerCancel: wx.ID_CANCEL,
        answerOK: wx.ID_OK
    }
    
    def _messagebox(self, type, caption, text,
                    button1, button2=None, button3=None):
        
        style = _wxicons[type]
        buttons = [button1, button2, button3]
        if (wx.ID_YES in buttons) or (wx.ID_NO in buttons):
            style = style | wx.YES_NO
        if wx.ID_OK in buttons:
            style = style | wx.OK
        if wx.ID_CANCEL in buttons:
            style = style | wx.CANCEL
        dlg = wx.MessageDialog(self._parent, text, caption, style)
        # Hack to fix strange button ordering for Yes/No/Cancel
        #b_cancel = dlg.FindWindowById(wx.ID_CANCEL)
        #b_no = dlg.FindWindowById(wx.ID_NO)
        #print b_cancel, b_no
        #if (b_cancel is not None) and (b_no is not None):
        #    b_yes = dlg.FindWindowById(wx.ID_YES)
        #    if b_yes:
        #        b_no.MoveAfterInTabOrder(b_yes)
        #    b_cancel.MoveAfterInTabOrder(b_no)
        result = dlg.ShowModal()
        dlg.Destroy()
        return result


class PWxFileDialog(app.PFileDialogBase):
    
    def _wxfiledialog(self, msg, path, filter, style):
        if filter == "":
            filter = "*"
        dlg = wx.FileDialog(self._parent, msg, path, "", filter, style)
        result = dlg.ShowModal()
        if result == wx.ID_OK:
            retval = dlg.GetPath()
        else:
            retval = ""
        dlg.Destroy()
        return retval
    
    def openfiledialog(self, path, filter):
        return self._wxfiledialog("Select file to open", path, filter,
                                  wx.FD_OPEN)
    
    def savefiledialog(self, path, filter):
        return self._wxfiledialog("Select file to save", path, filter,
                                  wx.FD_SAVE)


def _wx_icon_from_file(filename):
    # Kludgy but gets the job done on all platforms
    return wx.IconFromBitmap(wx.BitmapFromImage(wx.Image(filename)))


class PWxAboutDialog(app.PAboutDialogBase, wx.AboutDialogInfo):
    
    attrmap = {
        'name': "SetName",
        'version': "SetVersion",
        'copyright': "SetCopyright",
        'license': "SetLicense",
        'description': "SetDescription",
        'developers': "SetDevelopers",
        'website': "SetWebSite",
        'icon': "SetIconFromFile"
    }
    
    def __init__(self, parent):
        wx.AboutDialogInfo.__init__(self)
        app.PAboutDialogBase.__init__(self, parent)
    
    def SetIconFromFile(self, filename):
        self.SetIcon(_wx_icon_from_file(filename))
    
    def display(self):
        wx.AboutBox(self)


class _PWxBaseMixin(_PWxCommunicator, wx.Frame):
    """Mixin class for wxWidgets base windows.
    """
    
    def _setup_events(self):
        # 'automagic' connections
        self.setup_notify(SIGNAL_QUERYCLOSE, self.OnCloseWindow, False)
        self.setup_notify(SIGNAL_SHOWEVENT, self.OnShowEvent, False)
    
    def _get_w(self):
        return self.GetClientSizeTuple()[0]
    w = property(_get_w)
    
    def _show_window(self):
        wx.Frame.Show(self)
    
    def _hide_window(self):
        wx.Frame.Hide(self)
    
    def set_caption(self, caption):
        self.SetTitle(caption)
    
    def sizetoscreen(self, maximized):
        if maximized:
            self.Maximize(maximized)
        else:
            displayrect = wx.Display().GetClientArea()
            self.SetSizeWH(displayrect.GetWidth() - self.sizeoffset,
                           displayrect.GetHeight() - self.sizeoffset)
    
    def sizetoclient(self, clientwidth, clientheight):
        self.SetClientSizeWH(clientwidth, clientheight)
    
    def center(self):
        self.CenterOnScreen()
    
    def show_init(self):
        self.Show(True)
    
    def do_exit(self):
        self.Close()
    
    def OnShowEvent(self, event):
        # generates custom show/hide events from single wxWidgets show event
        if event.GetShow():
            self.do_notify(SIGNAL_SHOWN)
        else:
            self.do_notify(SIGNAL_HIDDEN)
    
    def OnCloseWindow(self, event):
        # 'automagic' method for SIGNAL_QUERYCLOSE
        if event.CanVeto() and not self._canclose():
            event.Veto()
        else:
            # Send the closing signal
            self.do_notify(SIGNAL_CLOSING)
            # Make sure the signal handlers are done before we destroy
            self.ProcessPendingEvents()
            self.Destroy()


class PWxBaseWindow(_PWxBaseMixin, app.PBaseWindowBase):
    """Customized wxWidgets base window class.
    """
    
    def __init__(self, parent, cls=None):
        _PWxBaseMixin.__init__(self, None)
        app.PBaseWindowBase.__init__(self, parent, cls)
        self._setup_events()
    
    def show_init(self):
        app.PBaseWindowBase.show_init(self)
        _PWxBaseMixin.show_init(self)


class _PWxMainMixin(_PWxBaseMixin):
    """Mixin class for wxWidgets top windows and main windows.
    """
    
    messageboxclass = PWxMessageBox
    filedialogclass = PWxFileDialog
    aboutdialogclass = PWxAboutDialog
    
    def set_iconfile(self, iconfile):
        self.SetIcon(_wx_icon_from_file(iconfile))
    
    def _size_to_settings(self, width, height):
        self.SetSizeWH(width, height)
        self.SetMinSize((width, height))
    
    def _move_to_settings(self, left, top):
        self.SetPosition(wx.Point(left, top))
    
    def _get_current_geometry(self):
        left, top = self.GetPosition()
        width, height = self.GetSizeTuple()
        return left, top, width, height
    
    def choose_directory(self, curdir):
        dlg = wx.DirDialog(self._parent, defaultPath=curdir)
        result = dlg.ShowModal()
        if result == wx.ID_OK:
            retval = dlg.GetPath()
        else:
            retval = ""
        dlg.Destroy()
        return retval


class PWxTopWindow(_PWxMainMixin, app.PTopWindowBase):
    """Customized wxWidgets top window class.
    """
    
    def __init__(self, parent, cls=None):
        _PWxMainMixin.__init__(self, None)
        app.PTopWindowBase.__init__(self, parent, cls)
        self._setup_events()
    
    def show_init(self):
        app.PTopWindowBase.show_init(self)
        _PWxMainMixin.show_init(self)


class PWxApplication(_PWxCommunicator, wx.App, app.PApplicationBase):
    """Customized wxWidgets application class.
    """
    
    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
        wx.App.__init__(self, arglist)
    
    def OnInit(self):
        # wxWidgets wants you to initialize subwidgets here
        
        self.mainwin = self.createMainWidget()
        self.SetTopWindow(self.mainwin)
        
        # required return value
        return True
    
    def OnExit(self):
        # 'automagic' method for SIGNAL_BEFOREQUIT
        self.before_quit()
    
    def _eventloop(self):
        self.MainLoop()
    
    def process_events(self):
        self.ProcessPendingEvents()
        #self.Yield()
        #wx.SafeYield()



#!/usr/bin/env python3
"""
Module TABWIDGET -- GUI Tab Widget
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for a tab widget.
"""

from plib.stdlib.coll import baselist

from plib.gui.defs import *
from plib.gui._widgets import _helpers
from plib.gui._widgets import _signal


class PTabWidgetBase(_signal._PNotifyControl, _helpers._PShift, baselist):
    """Tab widget as a Python list of 2-tuples (tab-title, widget).
    """
    
    signal = SIGNAL_TABCHANGED
    
    def __init__(self, parent, tabs=None, target=None):
        self._parent = parent
        _signal._PNotifyControl.__init__(self, target)
        _helpers._PShift.__init__(self)
        baselist.__init__(self)
        if tabs is not None:
            self._createtabs(tabs)
    
    def _createtabs(self, tabs):
        """Create tabs from sequence.
        
        Assumes that the sequence contains tuples of (title, widget)
        for each tab.
        """
        self.extend(tabs)
    
    def _get_tabtitle(self, index):
        """ Placeholder for derived classes to implement. """
        raise NotImplementedError
    
    def _set_tabtitle(self, index, title):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _add_tab(self, index, title, widget):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _del_tab(self, index):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _get_data(self, index):
        return (self._get_tabtitle(index), self._items[index])
    
    def _set_data(self, index, value):
        # This gets a little complicated because we have to delete the
        # old tab at this index and insert a new one.
        self._del_data(index)
        self._add_data(index, value)
    
    def _add_data(self, index, value):
        self._shiftright(index)
        self._items[index] = value[1]
        self._add_tab(index, value[0], value[1])
    
    def _del_data(self, index):
        self._del_tab(index)
        del self._items[index]
        self._shiftleft(index)
    
    def current_index(self):
        """Return the currently selected tab index.
        
        Derived classes must implement.
        """
        raise NotImplementedError
    
    def set_current_index(self, index):
        """Set the currently selected tab index.
        
        Derived classes must implement.
        """
        raise NotImplementedError
    
    def current_title(self):
        return self[self.current_index()][0]
    
    def current_widget(self):
        return self[self.current_index()][1]



#!/usr/bin/env python3
"""
Module QT4LISTVIEW -- Python Qt 4 Tree/List View Objects
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI objects for the tree/list view widgets.
"""

from PyQt4 import Qt as qt

from plib.gui._widgets import listview

from ._qt4common import (_PQtMeta, _PQtWidget, _PQtClientWidget, _PQtCommunicator,
                        _qtalignmap)


class PQtListViewItem(_PQtCommunicator, qt.QTreeWidgetItem,
                      listview.PListViewItemBase, metaclass=_PQtMeta):
    
    def __init__(self, parent, index, data=None):
        qt.QTreeWidgetItem.__init__(self, parent)
        parent._insert(self, index)
        listview.PListViewItemBase.__init__(self, parent, index, data)
    
    def _insert(self, item, index):
        if index == len(self):
            self.addChild(item)
        else:
            self.insertChild(index, item)
    
    def _helperdel(self, index, item):
        self.takeChild(index)
    
    def _get_col(self, col):
        return str(self.text(col))
    
    def _set_col(self, col, value):
        self.setText(col, str(value))
        # FIXME: it would be nice if this could be done once instead of
        # per item
        self.setTextAlignment(col,
                              self.listview.headerItem().textAlignment(col))
    
    def expand(self):
        self.setExpanded(True)


class PQtListViewLabels(listview.PListViewLabelsBase):
    
    label_list = None
    labels_initialized = False
    
    def _update(self, data):
        # Hack to get around weirdness in Qt 4 table widget API
        self.label_list = [str(value) for value in data]
        listview.PListViewLabelsBase._update(self, data)
    
    def _set_label(self, index, label):
        if self.label_list is not None:
            # First time setting labels, do it this way
            self.listview.setHeaderLabels(self.label_list)
            self.label_list = None
        if self.labels_initialized:
            # This allows labels to be changed after the initial setup
            item = self.listview.headerItem()
            item.setText(index, label)
        elif index == (len(self) - 1):
            # End of initial run
            self.labels_initialized = True
    
    def _set_width(self, index, width):
        if width > 0:
            self.listview.header().setResizeMode(
                index,
                qt.QHeaderView.Interactive
            )
            self.listview.header().resizeSection(index, width)
        else:
            self.listview.header().setResizeMode(
                index,
                qt.QHeaderView.ResizeToContents
            )
    
    def _set_align(self, index, align):
        item = self.listview.headerItem()
        item.setTextAlignment(index, _qtalignmap[align])
        # each item will align itself when added
    
    def _set_readonly(self, index, readonly):
        #self.table.setColumnReadOnly(index, readonly)
        pass


class _PQtListViewBase(qt.QTreeWidget):
    
    widget_class = qt.QTreeWidget
    
    itemclass = PQtListViewItem
    labelsclass = PQtListViewLabels
    
    def __init__(self, parent):
        qt.QTreeWidget.__init__(self, parent)
        self.header().setStretchLastSection(False)
        self.setSortingEnabled(False)
        self.setRootIsDecorated(True)
    
    def _insert(self, item, index):
        if index == len(self):
            self.addTopLevelItem(item)
        else:
            self.insertTopLevelItem(index, item)
    
    def _helperdel(self, index, item):
        self.takeTopLevelItem(index)
    
    def _set_header_font_object(self, font_name, font_size, bold, italic):
        self.header().setFont(self._qt_font_object(
            font_name, font_size, bold, italic))
    
    def colcount(self):
        return self.columnCount()
    
    def current_item(self):
        return self.currentItem()
    
    def set_current_item(self, item):
        self.setCurrentItem(item)


class PQtListView(_PQtClientWidget, _PQtListViewBase, listview.PListViewBase, metaclass=_PQtMeta):
    
    def __init__(self, parent, labels=None, data=None, target=None):
        _PQtListViewBase.__init__(self, parent)
        listview.PListViewBase.__init__(self, parent, labels, data, target)


class PQtListBox(_PQtWidget, _PQtListViewBase, listview.PListBoxBase, metaclass=_PQtMeta):
    
    def __init__(self, parent, labels=None, data=None,
                 target=None, geometry=None):
        
        _PQtListViewBase.__init__(self, parent)
        listview.PListBoxBase.__init__(self, parent, labels, data,
                                       target, geometry)



#!/usr/bin/env python3
"""
Module TABLE -- GUI Table Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for table widgets.
"""

from plib.gui.defs import *
from plib.gui._widgets import _helpers
from plib.gui._widgets import _signal

scrollbarsize = 20
gridlinesize = 2


class PTableRow(_helpers._PRowHelperItem):
    """Table row helper class.
    
    Looks like a list of strings (the values in the cells
    in this row).
    """
    
    def _get_table(self):
        return self._helper
    
    def _get_row(self):
        return self._i
    
    table = property(_get_table)
    row = property(_get_row)
    
    def __len__(self):
        return self.table.colcount()
    
    def _get_data(self, index):
        return self.table._get_cell(self._i, index)
    
    def _set_data(self, index, value):
        self.table._set_cell(self._i, index, value)


class PTableLabelsBase(_helpers._PHelperColLabels):
    """Table labels helper class.
    
    Note that the length of the list of labels passed to
    this class determines the column count of the table.
    """
    
    def _update(self, data):
        if self.table.colcount() != len(data):
            self.table.set_colcount(len(data))
        _helpers._PHelperColLabels._update(self, data)
    
    def _get_table(self):
        return self._helper
    
    table = property(_get_table)


class PTableBase(_signal._PNotifyControl, _helpers._PRowHelper):
    """Table class that looks like a list of PTableRows.
    
    Each PTableRow looks like a list of strings. Double-indexing
    the table by [row][col] therefore retrieves the string in
    cell row, col).
    """
    
    signal = SIGNAL_CELLCHANGED
    itemclass = PTableRow
    defaultcolwidth = 100
    defaultrowheight = 25
    defaultmargin = 25
    
    def __init__(self, parent, labels=None, data=None, target=None):
        self._parent = parent
        self._adding_row = self._deleting_row = False
        _signal._PNotifyControl.__init__(self, target)
        _helpers._PRowHelper.__init__(self, labels, data)
        
        # set minimum size after initializing data if there is data
        if data is not None:
            self.set_min_size(self.minwidth(), self.minheight())
        
        # enable signals after initializing data so they don't get
        # fired on initialization
        self.setup_notify(SIGNAL_TABLECHANGED, self._tablechanged)
    
    def _add(self, index, value):
        self._adding_row = True
        super(PTableBase, self)._add(index, value)
        self._adding_row = False
    
    def _del(self, index):
        self._deleting_row = True
        super(PTableBase, self)._del(index, value)
        self._deleting_row = False
    
    def _tablechanged(self, row, col):
        # Filter out the signals we're not interested in
        if (row == self.current_row()) and (col == self.current_col()):
            self.do_notify(SIGNAL_CELLCHANGED, row, col)
    
    def _get_cell(self, row, col):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _set_cell(self, row, col, value):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def colcount(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_colcount(self, count):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def current_col(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def current_cell(self):
        r = self.current_row()
        c = self.current_col()
        if (r < 0) or (c < 0):
            return None
        else:
            return self._items[r][c]
    
    def set_min_size(self, width, height):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def topmargin(self):
        return self.defaultmargin
    
    def leftmargin(self):
        return self.defaultmargin
    
    def rowheight(self, row):
        return self.defaultrowheight
    
    def colwidth(self, col):
        return self.defaultcolwidth
    
    def minwidth(self):
        return (
            sum(self.colwidth(col) for col in range(self.colcount())) +
            self.leftmargin() + scrollbarsize +
            (self.colcount() * gridlinesize)
        )
    
    def minheight(self):
        return (
            sum(self.rowheight(row) for row in range(self.rowcount())) +
            self.topmargin() + scrollbarsize +
            (self.rowcount() * gridlinesize)
        )
    
    def force_repaint(self):
        """Placeholder for derived classes to implement.
        """
        pass
    
    def default_fgcolor(self):
        return COLOR_BLACK
    
    def default_bkcolor(self):
        return COLOR_WHITE
    
    def set_text_fgcolor(self, row, col, color):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_cell_bkcolor(self, row, col, color):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_row_fgcolor(self, row, color):
        for col in range(self.colcount()):
            self.set_text_fgcolor(row, col, color)
        self.force_repaint()
    
    def set_row_bkcolor(self, row, color):
        for col in range(self.colcount()):
            self.set_cell_bkcolor(row, col, color)
        self.force_repaint()
    
    def set_col_fgcolor(self, col, color):
        for row in range(self.rowcount()):
            self.set_text_fgcolor(row, col, color)
        self.force_repaint()
    
    def set_col_bkcolor(self, col, color):
        for row in range(self.rowcount()):
            self.set_cell_bkcolor(row, col, color)
        self.force_repaint()



#!/usr/bin/env python3
"""
Module PYSIDE2TABLE -- Python PySide2 Table Objects
Sub-Package GUI.TOOLKITS.PYSIDE2 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for the table widgets.
"""

from PySide2 import QtGui as qtg, QtWidgets as qt

from plib.gui.defs import *
from plib.gui._widgets import table

from ._pyside2common import _PQtSequenceMeta, _PQtClientWidget, _qtalignmap


class PQtTableLabels(table.PTableLabelsBase):
    
    label_list = None
    labels_initialized = False
    
    def _update(self, data):
        # Hack to get around weirdness in PySide table widget API
        self.label_list = [str(value) for value in data]
        table.PTableLabelsBase._update(self, data)
    
    def _set_label(self, index, label):
        if self.label_list is not None:
            # First time setting labels, do it this way
            self.table.setHorizontalHeaderLabels(self.label_list)
            self.label_list = None
        if self.labels_initialized:
            # This allows labels to be changed after the initial setup
            item = self.table.horizontalHeaderItem(index)
            item.setText(label)
        elif index == (len(self) - 1):
            # End of initial run
            self.labels_initialized = True
    
    def _set_width(self, index, width):
        if width > 0:
            self.table.horizontalHeader().setSectionResizeMode(
                index,
                qt.QHeaderView.Interactive
            )
            self.table.horizontalHeader().resizeSection(index, width)
        else:
            self.table.horizontalHeader().setSectionResizeMode(
                index,
                qt.QHeaderView.ResizeToContents
            )
    
    def _set_align(self, index, align):
        self.table.horizontalHeaderItem(index).setTextAlignment(
            _qtalignmap[align])
        # Table cells will align themselves when added
    
    def _set_readonly(self, index, readonly):
        pass


class PQtTable(_PQtClientWidget, qt.QTableWidget, table.PTableBase, metaclass=_PQtSequenceMeta):
    
    labelsclass = PQtTableLabels
    
    event_signals = (SIGNAL_CELLCHANGED,)
    
    def __init__(self, parent, labels=None, data=None, target=None):
        qt.QTableWidget.__init__(self, parent)
        self.setSortingEnabled(False)
        # Used by ugly hack in default fg and bk color methods, below
        pal = self.palette()
        self._def_fgcolor = pal.color(self.foregroundRole())
        self._def_bkcolor = pal.color(self.backgroundRole())
        # Used by ugly hack in setup_notify, below
        self._setting_colors = False
        
        # This will initialize data (if any)
        table.PTableBase.__init__(self, parent, labels, data, target)
    
    def setup_notify(self, signal, target):
        # Hack to mask out table changed events fired when cell colors are
        # changed (the PySide API says this shouldn't happen but it does,
        # go figure)
        if signal == SIGNAL_TABLECHANGED:
            def _wrapper(row, col):
                if not self._setting_colors:
                    target(row, col)
            actual_target = _wrapper
        else:
            actual_target = target
        _PQtClientWidget.setup_notify(self, signal, actual_target)
    
    def _get_item(self, row, col):
        result = self.item(row, col)
        if not isinstance(result, qt.QTableWidgetItem):
            result = qt.QTableWidgetItem()
            self.setItem(row, col, result)
        return result
    
    def _set_header_font_object(self, font_name, font_size, bold, italic):
        self.horizontalHeader().setFont(self._qt_font_object(
            font_name, font_size, bold, italic))
    
    def _get_cell(self, row, col):
        return self._get_item(row, col).text()
    
    def _set_cell(self, row, col, value):
        item = self._get_item(row, col)
        item.setText(str(value))
        # FIXME: it would be nice if this could be done once instead of
        # per item
        item.setTextAlignment(self.horizontalHeaderItem(col).textAlignment())
    
    def rowcount(self):
        return self.rowCount()
    
    def colcount(self):
        return self.columnCount()
    
    def set_colcount(self, count):
        self.setColumnCount(count)
    
    def current_row(self):
        return self.currentRow()
    
    def current_col(self):
        return self.currentColumn()
    
    def _insert_row(self, index):
        self.insertRow(index)
    
    def _remove_row(self, index):
        self.removeRow(index)
    
    def set_min_size(self, width, height):
        self.setMinimumSize(width, height)
    
    def topmargin(self):
        return 0  # self.topMargin()
    
    def leftmargin(self):
        return 0  # self.leftMargin()
    
    def rowheight(self, row):
        return self.rowHeight(row)
    
    def colwidth(self, col):
        return self.columnWidth(col)
    
    def default_fgcolor(self):
        return self._def_fgcolor
    
    def default_bkcolor(self):
        return self._def_bkcolor
    
    def set_text_fgcolor(self, row, col, color):
        item = self._get_item(row, col)
        self._setting_colors = True
        item.setForeground(qtg.QBrush(self._mapped_color(color)))
        self._setting_colors = False
    
    def set_cell_bkcolor(self, row, col, color):
        item = self._get_item(row, col)
        self._setting_colors = True
        item.setBackground(qtg.QBrush(self._mapped_color(color)))
        self._setting_colors = False



#!/usr/bin/env python3
"""
Module PYSIDETABLE -- Python PySide Table Objects
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for the table widgets.
"""

from PySide import QtGui as qt

from plib.gui.defs import *
from plib.gui._widgets import table

from ._pysidecommon import _PQtSequenceMeta, _PQtClientWidget, _qtalignmap


class PQtTableLabels(table.PTableLabelsBase):
    
    label_list = None
    labels_initialized = False
    
    def _update(self, data):
        # Hack to get around weirdness in PySide table widget API
        self.label_list = [str(value) for value in data]
        table.PTableLabelsBase._update(self, data)
    
    def _set_label(self, index, label):
        if self.label_list is not None:
            # First time setting labels, do it this way
            self.table.setHorizontalHeaderLabels(self.label_list)
            self.label_list = None
        if self.labels_initialized:
            # This allows labels to be changed after the initial setup
            item = self.table.horizontalHeaderItem(index)
            item.setText(label)
        elif index == (len(self) - 1):
            # End of initial run
            self.labels_initialized = True
    
    def _set_width(self, index, width):
        if width > 0:
            self.table.horizontalHeader().setResizeMode(
                index,
                qt.QHeaderView.Interactive
            )
            self.table.horizontalHeader().resizeSection(index, width)
        else:
            self.table.horizontalHeader().setResizeMode(
                index,
                qt.QHeaderView.ResizeToContents
            )
    
    def _set_align(self, index, align):
        self.table.horizontalHeaderItem(index).setTextAlignment(
            _qtalignmap[align])
        # Table cells will align themselves when added
    
    def _set_readonly(self, index, readonly):
        pass


class PQtTable(_PQtClientWidget, qt.QTableWidget, table.PTableBase, metaclass=_PQtSequenceMeta):
    
    labelsclass = PQtTableLabels
    
    event_signals = (SIGNAL_CELLCHANGED,)
    
    def __init__(self, parent, labels=None, data=None, target=None):
        qt.QTableWidget.__init__(self, parent)
        self.setSortingEnabled(False)
        # Used by ugly hack in default fg and bk color methods, below
        pal = self.palette()
        self._def_fgcolor = pal.color(self.foregroundRole())
        self._def_bkcolor = pal.color(self.backgroundRole())
        # Used by ugly hack in setup_notify, below
        self._setting_colors = False
        
        # This will initialize data (if any)
        table.PTableBase.__init__(self, parent, labels, data, target)
    
    def setup_notify(self, signal, target):
        # Hack to mask out table changed events fired when cell colors are
        # changed (the PySide API says this shouldn't happen but it does,
        # go figure)
        if signal == SIGNAL_TABLECHANGED:
            def _wrapper(row, col):
                if not self._setting_colors:
                    target(row, col)
            actual_target = _wrapper
        else:
            actual_target = target
        _PQtClientWidget.setup_notify(self, signal, actual_target)
    
    def _get_item(self, row, col):
        result = self.item(row, col)
        if not isinstance(result, qt.QTableWidgetItem):
            result = qt.QTableWidgetItem()
            self.setItem(row, col, result)
        return result
    
    def _set_header_font_object(self, font_name, font_size, bold, italic):
        self.horizontalHeader().setFont(self._qt_font_object(
            font_name, font_size, bold, italic))
    
    def _get_cell(self, row, col):
        return self._get_item(row, col).text()
    
    def _set_cell(self, row, col, value):
        item = self._get_item(row, col)
        item.setText(str(value))
        # FIXME: it would be nice if this could be done once instead of
        # per item
        item.setTextAlignment(self.horizontalHeaderItem(col).textAlignment())
    
    def rowcount(self):
        return self.rowCount()
    
    def colcount(self):
        return self.columnCount()
    
    def set_colcount(self, count):
        self.setColumnCount(count)
    
    def current_row(self):
        return self.currentRow()
    
    def current_col(self):
        return self.currentColumn()
    
    def _insert_row(self, index):
        self.insertRow(index)
    
    def _remove_row(self, index):
        self.removeRow(index)
    
    def set_min_size(self, width, height):
        self.setMinimumSize(width, height)
    
    def topmargin(self):
        return 0  # self.topMargin()
    
    def leftmargin(self):
        return 0  # self.leftMargin()
    
    def rowheight(self, row):
        return self.rowHeight(row)
    
    def colwidth(self, col):
        return self.columnWidth(col)
    
    def default_fgcolor(self):
        return self._def_fgcolor
    
    def default_bkcolor(self):
        return self._def_bkcolor
    
    def set_text_fgcolor(self, row, col, color):
        item = self._get_item(row, col)
        self._setting_colors = True
        item.setForeground(qt.QBrush(self._mapped_color(color)))
        self._setting_colors = False
    
    def set_cell_bkcolor(self, row, col, color):
        item = self._get_item(row, col)
        self._setting_colors = True
        item.setBackground(qt.QBrush(self._mapped_color(color)))
        self._setting_colors = False



#!/usr/bin/env python3
"""
Module MAIN -- Top-Level GUI Module
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This is the main module for the GUI sub-package; its
namespace contains all the GUI classes and constants
and the runapp() function which should be the main function
of a GUI application. It can be imported as follows::

    import plib.gui.main

- or - ::

    from plib.gui import main

Note that this module uses the ModuleProxy class to avoid
importing *all* of the widget modules at once; instead that
class allows 'lazy' importing of the modules only when they
are first used. (Note also that this means that::

    from plib.gui.main import *

will *not* work.)

BTW, for those who think I'm doing this because I simply
can't help using black magic instead of ordinary code,
you're absolutely right. :) However, there actually are
other good reasons to do it, which are explained in the
docstring for the PLIB3.IO.CLASSES sub-package.
"""

import sys

from plib.stdlib.util import ModuleProxy
from plib.stdlib.imp import import_from_module

from plib.gui._toolkit import gui_test, gui_toolkit, toolkit

# Names dictionary that will be passed to ModuleProxy; we'll generate
# its entries below using data lists and helper functions that return
# callables which will load the required sub-modules dynamically on use

toolkit_dict = {}

if gui_test:
    # The _gui module should have been patched to include GUI classes
    del gui_toolkit, gui_test
    from plib.gui._gui import *
    gui_toolkit = 0

else:
    # Toolkit-specific entries for namespace dictionary
    
    def _toolkit_import(toolkit, modname, klassname):
        # Return class klassname from module modname in toolkit
        return import_from_module(
            'plib.gui._toolkits._{}._{}{}'.format(
                toolkit.lower(), toolkit.lower(), modname),
            # hack for Qt/KDE 4 and PySide to avoid massive class renaming
            'P{}{}'.format(toolkit.rstrip('245').replace("PySide", "Qt"), klassname))
    
    
    def toolkit_helper(toolkit, modname, klassname):
        def f():
            return _toolkit_import(toolkit, modname, klassname)
        f.__name__ = '{}_{}_{}'.format(toolkit, modname, klassname)
        return f
    
    
    # Got the hack, might as well use it...
    PApplication = toolkit_helper(toolkit, 'app', 'Application')()
    
    toolkit_list = [
        ('label',
            ['TextLabel']),
        ('display',
            ['TextDisplay']),
        ('button',
            ['Button', 'CheckBox']),
        ('combo',
            ['ComboBox']),
        ('editctrl',
            ['EditBox', 'EditControl']),
        ('groupbox',
            ['GroupBox']),
        ('table',
            ['TableLabels', 'Table']),
        ('listview',
            ['ListViewLabels', 'ListViewItem', 'ListView', 'ListBox']),
        ('tabwidget',
            ['TabWidget']),
        ('panel',
            ['Panel']),
        ('statusbar',
            ['StatusBar']),
        ('action',
            ['Menu', 'ToolBar', 'Action']),
        ('mainwin',
            ['MainWindow']),
        ('app',
            ['BaseWindow', 'MessageBox', 'FileDialog', 'TopWindow'])
    ]
    
    # Some items are only present in certain toolkits
    if toolkit in ('KDE4', 'Qt4', 'PySide', 'Qt5', 'PySide2'):
        toolkit_list.append(
            ('notify', ['SocketNotifier'])
        )
    
    toolkit_dict.update(
        ('P{}'.format(klassname), toolkit_helper(toolkit, modname, klassname))
        for modname, klassnames in toolkit_list
        for klassname in klassnames
    )
    
    del toolkit_helper


# 'Mixin' entries for namespace dictionary
def mixin_helper(klassname, bases):
    def f():
        baselist = [
            import_from_module('plib.gui.{}'.format(modname), basename)
            for modname, basename in bases
        ]
        result = type(klassname, tuple(baselist), {})
        
        # Hacks here to avoid even worse hacks elsewhere
        if klassname == 'PAutoPanel':
            result.baseclass = result
            result.panelclass = sys.modules[__name__].PPanel
        elif klassname in ('PAutoTabWidget', 'PAutoGroupBox'):
            result.panelclass = sys.modules[__name__].PAutoPanel
        
        return result
    f.__name__ = 'mixin_{}'.format(klassname)
    return f


mixin_list = [
    ('PTableEditor',
        [('_mixins', 'PTableMixin'), ('main', 'PEditor')]),
    ('PTreeEditor',
        [('_mixins', 'PTreeMixin'), ('main', 'PEditor')]),
    ('PTextEditor',
        [('_mixins', 'PTextMixin'), ('main', 'PEditor')]),
    ('PTextFileEditor',
        [('_mixins', 'PTextFileMixin'), ('main', 'PFileEditor')]),
    ('PAutoPanel',
        [('_mixins', 'PPanelMixin'), ('main', 'PPanel')]),
    ('PAutoTabWidget',
        [('_mixins', 'PTabMixin'), ('main', 'PTabWidget')]),
    ('PAutoGroupBox',
        [('_mixins', 'PGroupMixin'), ('main', 'PGroupBox')]),
    ('PAutoStatusBar',
        [('_mixins', 'PStatusMixin'), ('main', 'PStatusBar')])
]


if gui_test:
    
    # Remove the mixins that aren't supported in the test toolkit
    def is_widget(baseklass):
        # Hack to make sure 'non-widget' classes don't get removed;
        # this will need to be manually kept in sync with the
        # non-widgets in mixin_list above
        return (baseklass not in ('PEditor', 'PFileEditor'))
    
    
    removes = []
    for item in mixin_list:
        klassname, bases = item
        for modname, baseklass in bases:
            if ((modname == 'main') and is_widget(baseklass) and
               not hasattr(sys.modules[__name__], baseklass)):
                removes.append(item)
    for item in removes:
        mixin_list.remove(item)
    
    # Clean up the temporary names used
    del is_widget, removes, item, klassname, bases, modname, baseklass

toolkit_dict.update(
    (klassname, mixin_helper(klassname, bases))
    for klassname, bases in mixin_list
)


# 'Sorted' entries for namespace dictionary
def sorted_helper(toolkit, modname, klassname):
    def f():
        baseklass = _toolkit_import(toolkit, modname, klassname)
        mixin = import_from_module('plib.gui._sorted', '{}SortMixin'.format(klassname))
        return type('PSorted{}'.format(klassname), (mixin, baseklass), {})
    f.__name__ = 'sorted_{}'.format(klassname)
    return f


sorted_list = [
    ('combo',
        ['ComboBox']),
    ('listview',
        ['ListViewItem', 'ListView', 'ListBox'])
]

toolkit_dict.update(
    ('PSorted{}'.format(klassname), sorted_helper(toolkit, modname, klassname))
    for modname, klassnames in sorted_list
    for klassname in klassnames
)


# Generic 'widget' entries for namespace dictionary
def widget_helper(klassname, modname):
    def f():
        return import_from_module(
            'plib.gui._widgets.{}'.format(modname), klassname)
    f.__name__ = 'widget_{}_{}'.format(modname, klassname)
    return f


widget_list = [
    ('PTableRow', 'table'),
    ('PListViewItemCols', 'listview')
]

toolkit_dict.update(
    (klassname, widget_helper(klassname, modname))
    for klassname, modname in widget_list
)


# Standalone class entries for namespace dictionary
def class_helper(modname, klassname):
    def f():
        return import_from_module('plib.gui.{}'.format(modname), klassname)
    f.__name__ = '{}_{}'.format(modname.lstrip('_'), klassname)
    return f


class_list = [
    ('_classes', ['PTextFile']),
    ('_dialogs', ['PDialogBase', 'PPrefsDialog']),
    ('_edit', ['PEditor', 'PFileEditor']),
    ('_header', ['PHeaderLabel']),
    ('_panels', ['PMainPanel']),
    ('NotifierClient', ['NotifierClient'])
]

toolkit_dict.update(
    (klassname, class_helper(modname, klassname))
    for modname, klassnames in class_list for klassname in klassnames
)

# Now set up module proxy to 'lazy import' toolkit items as needed;
# note that we don't need to bind it to a variable in this module
# because it installs itself in sys.modules in our place (and that
# means we don't *want* to bind it to a variable here because that
# would create a circular reference); also note that we use the
# autodiscover keyword argument to prevent ModuleProxy from trying
# to automatically add all the modules in our package to the lazy
# loading namespace; we have already set up the namespace above
# (this also lets us use None for the path parameter, which otherwise
# would be an issue since this module is not a package)

ModuleProxy(__name__).init_proxy(__name__, None, globals(), locals(),
                                 names=toolkit_dict, autodiscover=False)

# Clean up namespace (and allow memory for helper items to be released,
# since they've done their job and we don't need them any more)
del ModuleProxy, widget_helper, class_helper, mixin_helper, toolkit_dict

# Utility function for running application

default_appclass = [PApplication]  # allows overriding if desired


def runapp(appclass=None, arglist=[], use_mainwindow=False):
    """Run a GUI application.
    
    Runs application with args ``arglist``:
    
    - if ``appclass`` is ``None``, run a 'bare' application (only
      useful for demo purposes, sort of like a "Hello World" app);
    
    - if ``appclass`` is a subclass of ``PApplication``, instantiates
      and runs it (this should be very rare as subclassing a widget will
      almost always suffice);
    
    - otherwise, pass ``appclass`` to the default application, which
      will determine whether it is a main widget class (and then
      just instantiate it as the app's main widget) or a client widget
      class (which it will insert into an instance of its default
      main widget class).
    
    The ``use_mainwindow`` argument determines the default main widget
    class; either ``PTopWindow`` (if it's ``False``, the default) or
    ``PMainWindow`` (if it's ``True``). This only comes into play if
    the appclass parameter is a client widget class. See the
    ``pyidserver-gui`` and ``scrips-edit`` example programs for typical
    usage.
    """
    
    if appclass is None:
        app = default_appclass[0](arglist,
                                  use_mainwindow=use_mainwindow)
    elif issubclass(appclass, PApplication):
        app = appclass(arglist,
                       use_mainwindow=use_mainwindow)
    else:
        app = default_appclass[0](arglist, appclass,
                                  use_mainwindow=use_mainwindow)
    app.run()



#!/usr/bin/env python3
"""
Module EDITCTRL -- GUI Editing Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for edit controls.
"""

from plib.gui.defs import *
from plib.gui._widgets import _control


class _PEditBase(_control._PDialogControl):
    """Base edit control class, defines standard API.
    """
    
    fn_get_text = None
    fn_set_text = None
    
    fn_get_readonly = None
    fn_set_readonly = None
    
    def _get_text(self):
        return getattr(self, self.fn_get_text)()
    
    def _set_text(self, value):
        getattr(self, self.fn_set_text)(value)
    
    edit_text = property(_get_text, _set_text)
    
    def _get_readonly(self):
        return getattr(self, self.fn_get_readonly)()
    
    def _set_readonly(self, value):
        getattr(self, self.fn_set_readonly)(value)
    
    readonly = property(_get_readonly, _set_readonly)


class PEditBoxBase(_PEditBase):
    """Base class for single-line input control.
    """
    
    signal = SIGNAL_EDITCHANGED


class PEditControlBase(_PEditBase):
    """Base class for multi-line edit control.
    """
    
    signal = SIGNAL_TEXTCHANGED
    
    def can_undo(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def can_redo(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def can_clip(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def can_paste(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def clear_edit(self):
        # Derived classes may implement a faster method
        self.edit_text = ""
    
    def undo_last(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def redo_last(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def select_all(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def delete_selected(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def copy_to_clipboard(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def cut_to_clipboard(self):
        # Derived classes may implement a faster method
        self.copy_to_clipboard()
        self.delete_selected()
    
    def paste_from_clipboard(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Module QT5COMBO -- Python Qt 5 Combo Box Widgets
Sub-Package GUI.TOOLKITS.QT5 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI objects for combo boxes.
"""

from PyQt5 import QtWidgets as qt

from plib.gui._widgets import combo

from ._qt5common import _PQtSequenceMeta, _PQtWidget


class PQtComboBox(qt.QComboBox, _PQtWidget, combo.PComboBoxBase, metaclass=_PQtSequenceMeta):
    
    widget_class = qt.QComboBox
    
    def __init__(self, parent, sequence=None, target=None, geometry=None):
        qt.QComboBox.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        combo.PComboBoxBase.__init__(self, sequence, target, geometry)
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt buttons don't appear to fully respect QSizePolicy.Fixed
        self.setMinimumWidth(width)
    
    def current_text(self):
        return str(self.currentText())
    
    # Note that there's no quick override for set_current_text in Qt; the
    # corresponding method to the above doesn't do what we want (it changes
    # the stored text in the combo instead of selecting the text we give it)
    
    def current_index(self):
        return self.currentIndex()
    
    def set_current_index(self, index):
        self.setCurrentIndex(index)
    
    def count(self, value):
        # Method name collision, we want it to be the Python sequence
        # count method here.
        return combo.PComboBoxBase.count(self, value)
    
    def _indexlen(self):
        # Let this method access the Qt combo box count method.
        return qt.QComboBox.count(self)
    
    def _get_data(self, index):
        return str(self.itemText(index))
    
    def _set_data(self, index, value):
        self.setItemText(index, str(value))
    
    def _add_data(self, index, value):
        self.insertItem(index, str(value))
    
    def _del_data(self, index):
        self.removeItem(index)



#!/usr/bin/env python3
"""
Module SIGNAL -- Base for GUI Signal Emitting Controls
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines an abstract base class for all signal-emitting control widgets.
"""

from plib.gui.common import font_args
from plib.gui._base import _notify


class _PNotifyControl(_notify._PNotifyBase):
    """Base class for controls that send event signals.
    
    A notify control can be passed a signal target in its
    constructor. Also defines the common widget API methods.
    """
    
    fn_enable_get = None
    fn_enable_set = None
    
    def update_widget(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_colors(self, fg=None, bg=None):
        if fg is not None:
            self.set_foreground_color(fg)
        if bg is not None:
            self.set_background_color(bg)
    
    def set_foreground_color(self, color):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_background_color(self, color):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_font(self, font_name=None,
                 font_size=None, bold=None, italic=None):
        
        self._set_font_object(*font_args(self,
                                         font_name,
                                         font_size,
                                         bold,
                                         italic))
    
    def get_font_name(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def get_font_size(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def get_font_bold(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def get_font_italic(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _set_font_object(self, font_name, font_size, bold, italic):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _get_enabled(self):
        return getattr(self, self.fn_enable_get)()
    
    def _set_enabled(self, value):
        getattr(self, self.fn_enable_set)(value)
    
    enabled = property(_get_enabled, _set_enabled)
    
    def set_focus(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Module WXTABLE -- Python wxWidgets Table Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for table table.
"""

import wx.grid

from plib.gui.defs import *
from plib.gui._widgets import table

from ._wxcommon import _PWxClientWidget


class PWxTableLabels(table.PTableLabelsBase):
    
    def _set_label(self, index, label):
        self.table.SetColLabelValue(index, label)
    
    def _set_width(self, index, width):
        self.table.SetColSize(index, width)
    
    def _set_align(self, index, align):
        pass
    
    def _set_readonly(self, index, readonly):
        pass


class PWxTable(_PWxClientWidget, wx.grid.Grid, table.PTableBase):
    
    labelsclass = PWxTableLabels
    
    def __init__(self, parent, labels=None, data=None, target=None):
        wx.grid.Grid.__init__(self, parent, -1)
        self._align = ALIGN_JUST  # used by PWxPanel to determine placement
        self._expand = True
        self.CreateGrid(0, 0)
        table.PTableBase.__init__(self, parent, labels, data, target)
    
    def _set_header_font_object(self, font_name, font_size, bold, italic):
        self.SetLabelFont(self._wx_font_object(
            font_name, font_size, bold, italic))
    
    def _get_cell(self, row, col):
        return self.GetCellValue(row, col)
    
    def _set_cell(self, row, col, value):
        self.SetCellValue(row, col, value)
    
    def rowcount(self):
        return self.GetNumberRows()
    
    def colcount(self):
        return self.GetNumberCols()
    
    def set_colcount(self, count):
        self.InsertCols(0, count)
    
    def current_row(self):
        return self.GetGridCursorRow()
    
    def current_col(self):
        return self.GetGridCursorCol()
    
    def _insert_row(self, index):
        self.InsertRows(index, 1)
    
    def _remove_row(self, index):
        self.DeleteRows(index, 1)
    
    def set_min_size(self, width, height):
        self.SetSizeHints(width, height)
    
    def topmargin(self):
        return self.GetColLabelSize()
    
    def leftmargin(self):
        return self.GetRowLabelSize()
    
    def rowheight(self, row):
        return self.GetRowSize(row)
    
    def colwidth(self, col):
        return self.GetColSize(col)
    
    def default_fgcolor(self):
        return self.GetDefaultCellTextColour()
    
    def default_bkcolor(self):
        return self.GetDefaultCellBackgroundColour()
    
    def set_text_fgcolor(self, row, col, color):
        self.SetCellTextColour(row, col, color)
    
    def set_cell_bkcolor(self, row, col, color):
        self.SetCellBackgroundColour(row, col, color)



#!/usr/bin/env python3
"""
Module NotifierClient
Sub-Package GUI of Package PLIB3
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the NotifierClient class. This is
a mixin class designed to allow an async socket I/O class
to multiplex its event loop with a GUI event loop. Due to
limitations in some GUI toolkits, this functionality is
implemented in two different ways, depending on the toolkit
in use:

- For Qt 4, PySide, and KDE 4, the PSocketNotifier class is present,
  and its functionality is used to allow the GUI event loop to
  respond to socket events. This is the desired approach.

- For wxWidgets, there is no straightforward way to
  make the GUI event loop "see" socket events; there are possible
  approaches involving threading, but these are complex and prone
  to brittleness. Instead, the kludgy but workable approach is
  taken of making the asnyc socket I/O ``select`` loop the "primary"
  one, and using the GUI application's ``process_events`` method
  to pump its events based on a ``select`` timeout.
"""

import asyncore
import types

from plib.stdlib.builtins import first

from plib.gui import main as gui

try:
    from plib.io.async import SocketDispatcher
except ImportError:
    # Dummy class that won't be in anybody's mro
    class SocketDispatcher(object):
        pass


class NotifierClientMeta(type):
    # Evil hack to choose the appropriate functionality depending on
    # whether we are mixing in with an asyncore channel or a plib.io one
    
    def __new__(meta, name, bases, attrs):
        # Note that the middle condition is needed because otherwise we will
        # get a NameError when we construct NotifierClient below!
        if bases and (bases[0] is not object) and (bases[0] is NotifierClient):
            # Find the type of I/O channel we're mixing in with
            channel_class = first(
                klass for klass in bases
                if issubclass(klass, (SocketDispatcher, asyncore.dispatcher))
            )
            if channel_class is None:
                raise TypeError("{} is not a valid I/O channel class!".format(cls.__name__))
            elif issubclass(channel_class, SocketDispatcher):
                from ._notifier_io import NotifierClientMixin
            elif issubclass(channel_class, asyncore.dispatcher):
                from ._notifier_asyncore import NotifierClientMixin
                attrs['dispatcher_class'] = channel_class
            else:
                # This should never happen!
                raise RuntimeError("Broken plib.builtins function: first")
            
            # The interpreter should detect a metaclass conflict before we even
            # get here, but we put in a check to make doubly sure
            assert issubclass(meta, type(channel_class))
            
            # We have to do it this way because there's no way to use a dynamically
            # determined base class list in a class statement; it would be nice if
            # Python allowed class NotifierClient(*bases), but it doesn't :-)
            bases = (NotifierClientMixin,) + bases
            
            newklass = meta(name, bases, attrs)
            assert issubclass(newklass, channel_class)
            return newklass
        
        else:
            # We aren't a first-level mixin, do things normally
            return type.__new__(meta, name, bases, attrs)


app_obj = None

if hasattr(gui, 'PSocketNotifier'):  # Qt 4/5, PySide, KDE 4
    
    from plib.gui.defs import *
    
    notify_methods = {
        NOTIFY_READ: ('readable', 'read'),
        NOTIFY_WRITE: ('writable', 'write')
    }
    
    
    class NotifierClient(object, metaclass=NotifierClientMeta):
        """Mixin class to multiplex async socket client with GUI event loop.
        
        This class is intended to be mixed in with an async socket client
        class; for example::
            
            class MyClient(NotifierClient, async.SocketClient):
                pass
        
        For most purposes this class functions as a "drop-in" mixin; no
        method overrides or other customization should be necessary (other
        than the obvious override of ``process_data`` to do something with
        data received from the socket).
        
        Note that the notifier client object must be instantiated
        *before* the application's event loop is started, or no
        socket events will be processed. (Possible places to do that are
        constructors for any client widget classes, or the app itself
        if you are defining your own app class; or in any methods that
        are called from those constructors, such as the ``_create_panels``
        method of a main panel.)
        
        Note also that we override the ``do_loop`` method to yield control
        back to the GUI event loop, and the ``check_done`` method to
        un-yield so the function that called ``do_loop`` (normally the
        ``client_communicate`` method) can return as it normally would.
        This allows user code to be written portably, so that it does not
        even need to know which event loop is actually running.
        """
        
        notifier_class = gui.PSocketNotifier
        notifiers = None
        
        def get_notifier(self, notify_type):
            sfn, nfn = notify_methods[notify_type]
            result = self.notifier_class(self, notify_type,
                                         getattr(self, sfn),
                                         getattr(asyncore, nfn))
            result.auto_enable = False  # we'll do the re-enable ourselves
            return result
        
        def init_notifiers(self):
            if not self.notifiers:
                self.notifiers = [
                    self.get_notifier(t)
                    for t in (NOTIFY_READ, NOTIFY_WRITE)
                ]
            self.check_notifiers()
        
        def check_notifiers(self):
            if self.notifiers:
                for notifier in self.notifiers:
                    notifier.set_enabled(notifier.select_fn())
        
        def del_notifiers(self):
            if self.notifiers:
                del self.notifiers[:]
        
        def _doyield(self):
            # Start a local instance of the GUI event loop
            # (NOTE: *not* to be called from user code!)
            app_obj.enter_yield()
        
        def _unyield(self):
            # Return from a local instance of the GUI event loop
            # (NOTE: *not* to be called from user code!)
            app_obj.exit_yield()
    
    
    class NotifierApplication(gui.PApplication):
        
        def createMainWidget(self):
            global app_obj
            app_obj = self
            return super(NotifierApplication, self).createMainWidget()


else:  # wxWidgets
    
    
    class NotifierClient(object, metaclass=NotifierClientMeta):
        """Mixin class to multiplex async socket client with GUI event loop.
        
        This class is intended to be mixed in with an async socket client
        class; for example::
            
            class MyClient(NotifierClient, async.SocketClient):
                pass
        
        For most purposes this class functions as a "drop-in" mixin; no
        method overrides or other customization should be necessary (other
        than the obvious override of ``process_data`` to do something with
        data received from the socket).
        
        Note that the notifier client object must be instantiated
        *before* the application's event loop is started, or no
        socket events will be processed. (Possible places to do that are
        constructors for any client widget classes, or the app itself
        if you are defining your own app class; or in any methods that
        are called from those constructors, such as the ``_create_panels``
        method of a main panel.)
        """
        
        poll_timeout = 0.1  # needs to be a short timeout to keep GUI snappy
        
        def set_app(self):
            if app_obj.notifier_client is None:
                app_obj.notifier_client = self
        
        def clear_app(self):
            if app_obj.notifier_client is self:
                app_obj.notifier_client = None
    
    
    class NotifierApplication(gui.PApplication):
        
        notifier_client = None
        
        def createMainWidget(self):
            global app_obj
            app_obj = self
            return super(NotifierApplication, self).createMainWidget()
        
        def _eventloop(self):
            """Use the async I/O loop with a timeout to process GUI events.
            """
            if self.notifier_client is not None:
                self.process_events()  # start with a clean slate
                self.notifier_client.do_loop(self.process_events)
                self.process_events()  # clear all events before exiting
            else:
                super(NotifierApplication, self)._eventloop()


gui.default_appclass[0] = NotifierApplication



#!/usr/bin/env python3
"""
Module WXLISTVIEW -- Python wxWidgets Tree/List View Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for the tree/list view widgets.
"""

import wx
from wx import gizmos

from plib.gui.defs import *
from plib.gui._widgets import listview

from ._wxcommon import _PWxClientWidget, _PWxWidget, _wxalignmap


class _PWxListViewMixin(object):
    """Mixin class to help handle auto-sizing of columns.
    """
    
    def _get_autocols(self):
        if hasattr(self, '_autocols'):
            return self._autocols
        return self.listview.autocols
    
    autocols = property(_get_autocols)
    
    def AutoSizeItems(self, col):
        for item in self:
            item._size_col(col)
            item.AutoSizeItems(col)


class PWxListViewItem(_PWxListViewMixin, listview.PListViewItemBase):
    
    def __init__(self, parent, index, data=None):
        if index == len(parent):
            before = None
        else:
            before = parent._items[index]
        # Ugly hack to postpone creating the wx tree item until _set_col below
        self._b = before
        listview.PListViewItemBase.__init__(self, parent, index, data)
    
    def _helperdel(self, index, item):
        self.listview.Delete(item._id)
    
    def _get_col(self, col):
        return self.listview.GetItemText(self._id, col)
    
    def _set_col(self, col, value):
        value = str(value)  # just in case
        if (col == 0) and not hasattr(self, '_id'):
            if self._b is not None:
                self._id = self.listview.InsertItem(
                    self._parent._id, self._b._id, value)
            else:
                self._id = self.listview.AppendItem(
                    self._parent._id, value)
            # this trick is to allow PWxListView.current_item to work
            self.listview.SetItemPyData(self._id, self)
            # ugly hack to clear up the instance namespace
            del self._b
        else:
            self.listview.SetItemText(self._id, value, col)
        self._size_col(col)
    
    def _size_col(self, col):
        if col in self.listview._autocols:
            self.listview.AutoSizeCol(col, self._get_col(col), self._depth)
    
    def expand(self):
        self.listview.Expand(self._id)


class PWxListViewLabels(listview.PListViewLabelsBase):
    
    def __init__(self, helper, labels=None):
        listview.PListViewLabelsBase.__init__(self, helper, labels)
        self.listview._id = self.listview.AddRoot("root")
    
    def _set_label(self, index, label):
        if (index == self.listview.colcount()):
            self.listview.AddColumn(label)
        else:
            self.listview.SetColumnText(index, label)
    
    def _set_width(self, index, width):
        self.listview.SetColumnWidth(index, width)
    
    def _set_align(self, index, align):
        # FIXME: this doesn't seem to work
        self.listview.SetColumnAlignment(index, _wxalignmap[align])
    
    def _set_readonly(self, index, readonly):
        pass


lv_style = wx.TR_HAS_BUTTONS | wx.TR_HIDE_ROOT | wx.TR_FULL_ROW_HIGHLIGHT


class _PWxListViewBase(gizmos.TreeListCtrl, _PWxListViewMixin):
    
    itemclass = PWxListViewItem
    labelsclass = PWxListViewLabels
    
    _depth_w = 30  # additional column width for each sub-level in tree
    
    def __init__(self, parent):
        # This will be seen by the autocols property from mixin class above
        self._autocols = {}
        gizmos.TreeListCtrl.__init__(self, parent,
                                     style=lv_style)
        self._align = ALIGN_JUST  # used by PWxPanel to determine placement
        self._expand = True
    
    def AutoSizeCol(self, col, newtext=None, newdepth=0):
        if newtext is None:
            newtext = self.GetColumnText(col)
        w = self.GetTextExtent(newtext)[0] + (newdepth * self._depth_w)
        if w > self.autocols[col]:
            self.autocols[col] = w
            gizmos.TreeListCtrl.SetColumnWidth(self, col, w)
    
    def AutoSizeCols(self, headeronly=False):
        for col in self.autocols:
            self.AutoSizeCol(col)
            if not headeronly:
                self.AutoSizeItems(col)
    
    def SetColumnWidth(self, index, width):
        # Override to handle auto resize for width -1
        if width > 0:
            if index in self.autocols:
                del self.autocols[index]
            gizmos.TreeListCtrl.SetColumnWidth(self, index, width)
        else:
            self.autocols[index] = self.GetColumnWidth(index)
            self.AutoSizeCol(index)
    
    def _helperdel(self, index, item):
        self.Delete(item._id)
    
    def _set_header_font_object(self, font_name, font_size, bold, italic):
        self.GetHeaderWindow().SetFont(self._wx_font_object(
            font_name, font_size, bold, italic))
        self.AutoSizeCols(True)
    
    def colcount(self):
        return self.GetColumnCount()
    
    def current_item(self):
        # the item pointer was stored in the wx tree item's PyData above
        return self.GetItemPyData(self.GetSelection())
    
    def set_current_item(self, item):
        self.SelectItem(item._id)


class PWxListView(_PWxClientWidget, _PWxListViewBase, listview.PListViewBase):
    
    def __init__(self, parent, labels=None, data=None, target=None):
        _PWxListViewBase.__init__(self, parent)
        listview.PListViewBase.__init__(self, parent, labels, data, target)
        if labels is not None:
            self.Expand(self._id)


class PWxListBox(_PWxWidget, _PWxListViewBase, listview.PListBoxBase):
    
    def __init__(self, parent, labels=None, data=None,
                 target=None, geometry=None):
        
        _PWxListViewBase.__init__(self, parent)
        listview.PListBoxBase.__init__(self, parent, labels, data,
                                       target, geometry)
        if labels is not None:
            self.Expand(self._id)



#!/usr/bin/env python3
"""
Module QT5MAINWIN -- Python Qt 5 Main Window Objects
Sub-Package GUI.TOOLKITS.QT5 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI main window objects.
"""

from plib.gui.defs import *
from plib.gui._base import mainwin

from ._qt5common import _PQtWidgetMeta
from ._qt5app import _PQtMainMixin
from ._qt5action import PQtMenu, PQtToolBar, PQtAction
from ._qt5statusbar import PQtStatusBar


class PQtMainWindow(_PQtMainMixin, mainwin.PMainWindowBase, metaclass=_PQtWidgetMeta):
    """Customized Qt main window class.
    """
    
    menuclass = PQtMenu
    toolbarclass = PQtToolBar
    statusbarclass = PQtStatusBar
    actionclass = PQtAction
    
    def __init__(self, parent, cls=None):
        _PQtMainMixin.__init__(self)
        mainwin.PMainWindowBase.__init__(self, parent, cls)
        self.abouttoolkitfunc = self.app.aboutQt
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        mainwin.PMainWindowBase.show_init(self)
        _PQtMainMixin.show_init(self)



#!/usr/bin/env python3
"""
Module PYSIDE2APP -- Python PySide2 Application Objects
Sub-Package GUI.TOOLKITS.PYSIDE2 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI application objects.
"""

from PySide2 import QtCore as qtc, QtGui as qtg, QtWidgets as qt

from plib.gui.defs import *
from plib.gui._base import app

from ._pyside2common import _PQtCommunicator, _PQtWidgetMeta, _qtmessagefuncs


def _int(button):
    if button is not None:
        return button
    else:
        return 0


class PQtMessageBox(app.PMessageBoxBase):
    """Customized Qt message box.
    """
    
    questionmap = {
        answerYes: qt.QMessageBox.Yes,
        answerNo: qt.QMessageBox.No,
        answerCancel: qt.QMessageBox.Cancel,
        answerOK: qt.QMessageBox.Ok
    }
    
    def _messagebox(self, type, caption, text,
                    button1, button2=None, button3=None):
        
        buttons = default = button1
        if button2 is not None:
            buttons |= button2
            if button3 is not None:
                buttons |= button3
                default = button3
            else:
                default = button2
        
        return _qtmessagefuncs[type](self._parent, caption, text,
                                     buttons, default)


class PQtFileDialog(app.PFileDialogBase):
    
    def openfiledialog(self, path, filter):
        return qt.QFileDialog.getOpenFileName(None, "Open",
                                              path, filter)[0]
    
    def savefiledialog(self, path, filter):
        return qt.QFileDialog.getSaveFileName(None, "Save",
                                              path, filter)[0]


class PQtAboutDialog(app.PAboutDialogBase):
    
    attrnames = [
        'name',
        'version',
        'copyright',
        'license',
        'description',
        'developers',
        'website',
        'icon'
    ]
    
    formatstr = "{aname} {aversion}\n{adescription}\n{acopyright}\n{adevelopers}\n{awebsite}"
    
    def __getattribute__(self, name):
        # Here we have to modify the hack somewhat
        attrnames = object.__getattribute__(self, 'attrnames')
        if name in attrnames:
            object.__getattribute__(self, '__dict__')['temp'] = name
            name = 'store'
        return object.__getattribute__(self, name)
    
    def store(self, data):
        name = self.temp
        del self.temp
        if name == 'developers':
            data = ", ".join(data)
        setattr(self, "a{}".format(name), data)
    
    def display(self):
        caption = "About {}".format(self.aname)
        # Leave out icon here (setting it in set_iconfile below is enough, and
        # including it here will raise an exception if there is no icon)
        body = self.formatstr.format(**dict(
            ("a{}".format(name), getattr(self, "a{}".format(name)))
            for name in self.attrnames if name != 'icon'))
        qt.QMessageBox.about(self.mainwidget, caption, body)


class _PQtBaseMixin(_PQtCommunicator, qt.QMainWindow):
    """Mixin class for Qt base windows.
    """
    
    event_signals = (SIGNAL_CLOSING, SIGNAL_SHOWN, SIGNAL_HIDDEN)
    
    def _get_w(self):
        return self.width()
    w = property(_get_w)
    
    def _show_window(self):
        qt.QMainWindow.show(self)
    
    def _hide_window(self):
        qt.QMainWindow.hide(self)
    
    def set_caption(self, caption):
        self.setWindowTitle(caption)
    
    def _get_desktop_rect(self, primary=True):
        # Correctly handle virtual desktop across multiple screens
        desktop = self.app.desktop()
        l = desktop.x()
        t = desktop.y()
        w = desktop.width()
        h = desktop.height()
        if desktop.isVirtualDesktop() and primary:
            # Default to centering on the primary screen
            i = desktop.primaryScreen()
            n = desktop.numScreens()
            w = w / n
            # NOTE: We have to check for i > 0 here because in some
            # cases (e.g., when running in a VirtualBox), Qt thinks
            # the desktop is "virtual" but there's only one screen and
            # desktop.primaryScreen returns 0 instead of 1.
            if i > 0:
                l += w * (i - 1)
        else:
            i = 0
            n = 1
        return i, n, l, t, w, h
    
    def sizetoscreen(self, maximized):
        if maximized:
            if self.shown:
                self.showMaximized()
            else:
                self._showMax = True
        else:
            i, n, l, t, w, h = self._get_desktop_rect()
            self.resize(
                w - self.sizeoffset,
                h - self.sizeoffset)
            self.move(l, t)
    
    def sizetoclient(self, clientwidth, clientheight):
        self.resize(clientwidth, clientheight)
    
    def center(self):
        i, n, l, t, w, h = self._get_desktop_rect()
        s = self.frameSize()  # FIXME: this appears to give wrong values!
        x, y = s.width(), s.height()
        self.move(l + (w - x) / 2, t + (h - y) / 2)
    
    def show_init(self):
        if hasattr(self, '_showMax'):
            self.showMaximized()
            del self._showMax
        else:
            qt.QMainWindow.show(self)
    
    def do_exit(self):
        self.close()
    
    def closeEvent(self, event):
        # 'automagic' code for SIGNAL_QUERYCLOSE
        if self._canclose():
            self._emit_event(SIGNAL_CLOSING)
            event.accept()
        else:
            event.ignore()
    
    def showEvent(self, event):
        self._emit_event(SIGNAL_SHOWN)
    
    def hideEvent(self, event):
        self._emit_event(SIGNAL_HIDDEN)


class PQtBaseWindow(_PQtBaseMixin, app.PBaseWindowBase, metaclass=_PQtWidgetMeta):
    """Customized Qt base window class.
    """
    
    def __init__(self, parent, cls=None):
        _PQtBaseMixin.__init__(self)
        app.PBaseWindowBase.__init__(self, parent, cls)
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        app.PBaseWindowBase.show_init(self)
        _PQtBaseMixin.show_init(self)


class _PQtMainMixin(_PQtBaseMixin):
    """Mixin class for Qt top windows and main windows.
    """
    
    messageboxclass = PQtMessageBox
    filedialogclass = PQtFileDialog
    aboutdialogclass = PQtAboutDialog
    
    def set_iconfile(self, iconfile):
        self.setWindowIcon(qtg.QIcon(qtg.QPixmap(iconfile)))
    
    def _size_to_settings(self, width, height):
        self.resize(width, height)
    
    def _move_to_settings(self, left, top):
        self.move(left, top)
    
    def _get_current_geometry(self):
        p = self.pos()
        s = self.size()
        return p.x(), p.y(), s.width(), s.height()
    
    def choose_directory(self, curdir):
        return str(qt.QFileDialog.getExistingDirectory(
            self, "Select Folder", curdir))


class PQtTopWindow(_PQtMainMixin, app.PTopWindowBase, metaclass=_PQtWidgetMeta):
    """Customized Qt top window class.
    """
    
    def __init__(self, parent, cls=None):
        _PQtMainMixin.__init__(self)
        app.PTopWindowBase.__init__(self, parent, cls)
        self.abouttoolkitfunc = self.app.aboutQt
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        app.PTopWindowBase.show_init(self)
        _PQtMainMixin.show_init(self)


class PQtApplication(_PQtCommunicator, qt.QApplication, app.PApplicationBase):
    """Customized Qt application class.
    """
    
    _local_loop = None
    
    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
        qt.QApplication.__init__(self, arglist)
        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
        self.mainwin = self.createMainWidget()
        #self.setMainWidget(self.mainwin)
        
        # 'automagic' signal connection
        self.setup_notify(SIGNAL_BEFOREQUIT, self.before_quit)
    
    def _eventloop(self):
        self.exec_()
    
    def process_events(self):
        self.processEvents()
    
    # For use when multiplexing with other event types (e.g.,
    # in a NotifierClient
    
    def enter_yield(self):
        if self._local_loop is None:
            self._local_loop = qtc.QEventLoop()
            self._local_loop.exec_()
    
    def exit_yield(self):
        if self._local_loop is not None:
            self._local_loop.exit()
            del self._local_loop



#!/usr/bin/env python3
"""
Module STATUSBAR -- GUI Status Bar Classes
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the status bar class.
"""


class PStatusBarBase(object):
    """Base class for status bar widget.
    
    A standard status bar with a text area on the left and an
    area for custom widgets on the right.
    
    The widgets parameter passed to the constructor should be a
    sequence of callables, each of which returns a 2-tuple: the
    first element should be an attribute name, under which the
    widget will be stored in the status bar instance, and the
    second element should be a widget suitable for insertion into
    the status bar.
    """
    
    textareaclass = None
    
    def __init__(self, mainwin, widgets=None):
        self._mainwin = mainwin
        self.create_textarea()
        self._init_widgets(widgets)
    
    def _init_widgets(self, widgets):
        if widgets is not None:
            for widget in widgets:
                self.append_widget(widget())
    
    def _add_widget(self, widget, expand=False, custom=True):
        """Placeholder for derived classes to implement
        """
        raise NotImplementedError
    
    def append_widget(self, wname, widget):
        setattr(self, wname, widget)
        self._add_widget(widget)
    
    def create_textarea(self):
        self.textarea = None
        if self.textareaclass is not None:
            self.textarea = self.textareaclass(self)
            self._add_widget(self.textarea, True, False)
    
    def get_text(self):
        return self.textarea.get_text()
    
    def set_text(self, value):
        self.textarea.set_text(value)



#!/usr/bin/env python3
"""
Module DISPLAY -- GUI Text Display Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for text display widgets.
"""

from plib.gui._widgets import _control


class PTextDisplayBase(_control._PDialogControl):
    """Base class for widget that displays multiple lines of text.
    """
    
    def __init__(self, text=None, geometry=None):
        _control._PDialogControl.__init__(self, None, geometry)
        if text is not None:
            self.set_text(text)
    
    def get_text(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_text(self, value):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Internal Module _TOOLKIT
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the code to determine the string
variable ``toolkit`` based on the GUI toolkit being
used. This variable is used in various places in the
GUI library.
"""

from plib.gui.defs import GUI_WX, GUI_KDE4, GUI_QT4, GUI_PYSIDE, GUI_QT5, GUI_PYSIDE2
from plib.gui._gui import gui_test

if gui_test:
    gui_toolkit = 0
else:
    from plib.gui._gui import gui_toolkit

# Set the toolkit string
# TODO: modularize this so we only need to change one module for a new toolkit

if gui_toolkit == GUI_WX:
    toolkit = 'Wx'

elif gui_toolkit == GUI_KDE4:
    toolkit = 'KDE4'

elif gui_toolkit == GUI_QT4:
    toolkit = 'Qt4'

elif gui_toolkit == GUI_PYSIDE:
    toolkit = 'PySide'

elif gui_toolkit == GUI_QT5:
    toolkit = 'Qt5'

elif gui_toolkit == GUI_PYSIDE2:
    toolkit = 'PySide2'

elif gui_test:
    toolkit = 'Test'

else:
    raise ValueError("No GUI toolkit found; cannot run GUI application.")

del GUI_WX, GUI_KDE4, GUI_QT4, GUI_PYSIDE, GUI_QT5, GUI_PYSIDE2



#!/usr/bin/python3 -u
"""
Setup script for PLIB3.GUI package
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.gui import __version__ as version

name = "plib3.gui"
description = "A simple Python 3 GUI framework."

author = "Peter A. Donis"
author_email = "peterdonis@alum.mit.edu"

startline = 5
endspec = "The Zen of PLIB3"

dev_status = "Beta"

license = "GPLv2"

data_dirs = ["examples"]

classifiers = """
Environment :: Console
Environment :: MacOS X
Environment :: Win32 (MS Windows)
Environment :: X11 Applications :: GTK
Environment :: X11 Applications :: KDE
Environment :: X11 Applications :: Qt
Intended Audience :: Developers
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: POSIX :: Linux
Programming Language :: Python :: 3
Topic :: Software Development :: Libraries :: Python Modules
"""

install_requires = """
plib3.stdlib (>=0.9.24)
"""

post_install = ["plib3-setup-{}".format(s) for s in ("gui-examples", "gui")]

rst_header_template = """**{basename}** for {name} {version}

:Author:        {author}
:Release Date:  {releasedate}
"""


if __name__ == '__main__':
    import sys
    import os
    from subprocess import call
    from distutils.core import setup
    from setuputils import convert_rst, current_date, setup_vars, long_description as make_long_description
    
    if os.path.isfile("README.rst"):
        startline = 3
        long_description = make_long_description(globals(), filename="README.rst")
    if "sdist" in sys.argv:
        convert_rst(rst_header_template,
            startline=2,
            name=name.upper(),
            version=version,
            author=author,
            releasedate=current_date("%d %b %Y")
        )
        call(['sed', '-i', 's/gitlab.com\/pdonis\/plib-/pypi.python.org\/pypi\/plib./', 'README'])
        call(['sed', '-i', 's/gitlab.com\/pdonis\/plib3-/pypi.python.org\/pypi\/plib3./', 'README'])
        call(['sed', '-i', 's/gitlab.com\/pdonis/pypi.python.org\/pypi/', 'README'])
    setup(**setup_vars(globals()))
    if "install" in sys.argv:
        for scriptname in post_install:
            os.system(scriptname)



#!/usr/bin/env python3
"""
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains the Qt 4 toolkit modules.
"""



#!/usr/bin/env python3
"""
Module BUTTON -- GUI Button Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for button widgets.
"""

from plib.gui.defs import *
from plib.gui._widgets import _control


class PButtonBase(_control._PDialogControl):
    """Base class for push button.
    """
    
    signal = SIGNAL_CLICKED
    
    def __init__(self, caption=None, pxname=None,
                 target=None, geometry=None):
        
        if isinstance(caption, int):
            # 'caption' is actually an action key
            if pxname == "":
                pxname = caption
            caption = self.get_menu_str(caption)
        if isinstance(pxname, int):
            # 'pxname' is actually an action key
            pxname = self.get_icon_filename(pxname)
        if caption:
            self.set_caption(caption)
        if pxname:
            self.set_icon(pxname)
        _control._PDialogControl.__init__(self, target, geometry)
    
    def set_caption(self, caption):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_icon(self, pxname):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError


class PCheckBoxBase(PButtonBase):
    """ Base class for checkbox. """
    
    signal = SIGNAL_TOGGLED
    
    fn_get_checked = None
    fn_set_checked = None
    
    def __init__(self, caption=None, pxname=None,
                 tristate=False, target=None, geometry=None):
        
        PButtonBase.__init__(self, caption, pxname, target, geometry)
        if tristate:
            self.make_tristate()
    
    def make_tristate(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _get_checked(self):
        return getattr(self, self.fn_get_checked)()
    
    def _set_checked(self, value):
        getattr(self, self.fn_set_checked)(value)
    
    checked = property(_get_checked, _set_checked)



#!/usr/bin/env python3
"""
Module PYSIDECOMMON -- Python PySide Common Objects
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common PySide GUI objects for use by the other
PySide modules.
"""

from abc import ABCMeta
from itertools import chain

from PySide import QtGui as qt, QtCore as qtc

from plib.gui.defs import *

_qtalignmap = {
    ALIGN_LEFT: qtc.Qt.AlignLeft | qtc.Qt.AlignVCenter,
    ALIGN_CENTER: qtc.Qt.AlignCenter,
    ALIGN_RIGHT: qtc.Qt.AlignRight | qtc.Qt.AlignVCenter
}

_qtcolormap = dict(
    (color, qt.QColor(color.lower()))
    for color in COLORNAMES
)

_qtmessagefuncs = {
    MBOX_INFO: qt.QMessageBox.information,
    MBOX_WARN: qt.QMessageBox.warning,
    MBOX_ERROR: qt.QMessageBox.critical,
    MBOX_QUERY: qt.QMessageBox.question
}

_qtsignalmap = {
    SIGNAL_ACTIVATED: "triggered",
    SIGNAL_CLICKED: "clicked",
    SIGNAL_TOGGLED: "toggled",
    SIGNAL_SELECTED: "activated",
    SIGNAL_LISTSELECTED: "currentItemChanged",
    SIGNAL_CELLSELECTED: "currentCellChanged",
    SIGNAL_TABLECHANGED: "cellChanged",
    SIGNAL_TEXTCHANGED: "textChanged",  # textEdited()?
    SIGNAL_EDITCHANGED: "textChanged",  # textEdited(const QString&)?
    SIGNAL_ENTER: "returnPressed",
    SIGNAL_TABCHANGED: "currentChanged",
    SIGNAL_NOTIFIER: "activated",
    SIGNAL_BEFOREQUIT: "aboutToQuit"
}

_qteventmap = {
    SIGNAL_CELLCHANGED: ("sig_cellChanged", (int, int)),
    SIGNAL_TEXTSTATECHANGED: ("sig_textStateChanged", ()),
    SIGNAL_FOCUS_IN: ("sig_focusInEvent", ()),
    SIGNAL_FOCUS_OUT: ("sig_focusOutEvent", ()),
    SIGNAL_CLOSING: ("sig_closeEvent", ()),
    SIGNAL_SHOWN: ("sig_showEvent", ()),
    SIGNAL_HIDDEN: ("sig_hideEvent", ())
}


def _qtmap(signal):
    if signal in _qtsignalmap:
        return _qtsignalmap[signal]
    elif signal in _qteventmap:
        return _qteventmap[signal][0]
    else:
        return None


# Ugly hacks to fix metaclass conflict for classes that use the collection
# ABCs, and to allow auto-construction of event signals without a lot of
# boilerplate code in each widget class

_QtMeta = type(qtc.QObject)


def _setup_signals(bases, attrs):
    for signal in chain(
        attrs.get('event_signals', ()),
        *(getattr(base, 'event_signals', ()) for base in bases)
    ):
        signame, args = _qteventmap.get(signal)
        if signame:
            attrs[signame] = qtc.Signal(*args, name=signame)


class _PQtWidgetMeta(_QtMeta):
    # Metaclass for non-sequence widgets
    
    def __new__(meta, name, bases, attrs):
        _setup_signals(bases, attrs)
        return super(_PQtWidgetMeta, meta).__new__(meta, name, bases, attrs)


class _PQtSequenceMeta(ABCMeta, _QtMeta):
    # Metaclass for sequence widgets
    
    def __new__(meta, name, bases, attrs):
        _setup_signals(bases, attrs)
        return super(_PQtSequenceMeta, meta).__new__(meta, name, bases, attrs)
    
    def __init__(cls, name, bases, attrs):
        _QtMeta.__init__(cls, name, bases, attrs)
        ABCMeta.__init__(cls, name, bases, attrs)


class _PQtCommunicator(object):
    """Mixin class to abstract signal/slot functionality in Qt.
    """
    
    def setup_notify(self, signal, target):
        sig = _qtmap(signal)
        if sig is not None:
            try:
                getattr(self, sig).connect(target)
            except AttributeError:
                pass
    
    def do_notify(self, signal, *args):
        sig = _qtmap(signal)
        if sig is not None:
            try:
                getattr(self, sig).emit(*args)
            except AttributeError:
                pass
    
    _emit_event = do_notify


class _PQtWidgetBase(object):
    """Mixin class to provide common Qt widget methods.
    """
    
    fn_enable_get = 'isEnabled'
    fn_enable_set = 'setEnabled'
    
    _palette = None
    
    def update_widget(self):
        self.update()
    
    def preferred_width(self):
        return max(self.minimumSize().width(), self.sizeHint().width())
    
    def preferred_height(self):
        return max(self.minimumSize().height(), self.sizeHint().height())
    
    def set_width(self, width):
        self.resize(width, self.height())
    
    def set_height(self, height):
        self.resize(self.width(), height)
    
    def set_position(self, left, top):
        if not (None in (left, top)):
            self.move(left, top)
    
    def _mapped_color(self, color):
        if isinstance(color, qt.QColor):
            return color
        return _qtcolormap[color]
    
    def set_colors(self, fg=None, bg=None):
        palette = qt.QPalette(self.palette())
        if fg is not None:
            palette.setColor(self.foregroundRole(), self._mapped_color(fg))
        if bg is not None:
            self.setAutoFillBackground(True)
            palette.setColor(self.backgroundRole(), self._mapped_color(bg))
        self.setPalette(palette)
    
    def set_foreground_color(self, color):
        self.set_colors(fg=color)
    
    def set_background_color(self, color):
        self.set_colors(bg=color)
    
    def get_font_name(self):
        return self.font().family()
    
    def get_font_size(self):
        return self.font().pointSize()
    
    def get_font_bold(self):
        return self.font().bold()
    
    def get_font_italic(self):
        return self.font().italic()
    
    def _qt_font_object(self, font_name, font_size, bold, italic):
        font = qt.QFont(font_name, font_size)
        font.setBold(bold)
        font.setItalic(italic)
        return font
    
    def _set_font_object(self, font_name, font_size, bold, italic):
        self.setFont(self._qt_font_object(
            font_name, font_size, bold, italic))
    
    def set_focus(self):
        self.setFocus()


class _PQtWidget(_PQtCommunicator, _PQtWidgetBase):
    """Mixin class for Qt widgets that can send/receive signals.
    """
    
    event_signals = (SIGNAL_FOCUS_IN, SIGNAL_FOCUS_OUT)
    
    def focusInEvent(self, event):
        super(_PQtWidget, self).focusInEvent(event)
        self._emit_event(SIGNAL_FOCUS_IN)
    
    def focusOutEvent(self, event):
        super(_PQtWidget, self).focusOutEvent(event)
        self._emit_event(SIGNAL_FOCUS_OUT)


class _PQtClientWidget(_PQtWidget):
    """Mixin class for Qt main window client widgets.
    """
    pass



#!/usr/bin/env python3
"""
Module WXLABEL -- Python wxWidgets Text Label Widgets
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for text label widgets.
"""

import wx

from plib.gui.defs import *
from plib.gui._widgets import label

from ._wxcommon import _PWxWidget


class PWxTextLabel(_PWxWidget, wx.StaticText, label.PTextLabelBase):
    
    def __init__(self, parent, text=None, geometry=None):
        wx.StaticText.__init__(self, parent)
        self._align = ALIGN_LEFT
        label.PTextLabelBase.__init__(self, text, geometry)
    
    def get_text(self):
        return self.GetLabel()
    
    def set_text(self, value):
        self.SetLabel(value)



#!/usr/bin/env python3
"""
Module PYSIDE2BUTTON -- Python PySide2 Button Widgets
Sub-Package GUI.TOOLKITS.PYSIDE2 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for button widgets.
"""

from PySide2 import QtGui as qtg, QtWidgets as qt

from plib.gui._widgets import button

from ._pyside2common import _PQtWidgetMeta, _PQtWidget


class _PQtButtonMixin(object):
    
    def set_caption(self, caption):
        self.setText(caption)


class PQtButton(_PQtButtonMixin, _PQtWidget, qt.QPushButton,
                button.PButtonBase, metaclass=_PQtWidgetMeta):
    
    def __init__(self, parent, caption=None, pxname=None,
                 target=None, geometry=None):
        
        qt.QPushButton.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        button.PButtonBase.__init__(self, caption, pxname, target, geometry)
    
    def set_icon(self, pxname):
        self.setIcon(qtg.QIcon(qtg.QPixmap(pxname)))
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt buttons don't appear to fully respect QSizePolicy.Fixed
        self.setMinimumWidth(width)


class PQtCheckBox(_PQtButtonMixin, _PQtWidget, qt.QCheckBox,
                  button.PCheckBoxBase, metaclass=_PQtWidgetMeta):
    
    fn_get_checked = 'isChecked'
    fn_set_checked = 'setChecked'
    
    def __init__(self, parent, caption=None, pxname=None, tristate=False,
                 target=None, geometry=None):
        
        qt.QCheckBox.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        button.PCheckBoxBase.__init__(self, caption, pxname, tristate,
                                      target, geometry)
    
    def set_icon(self, pxname):
        # FIXME: pixmaps on checkboxes?
        pass
    
    def make_tristate(self):
        self.setTriState(True)



#!/usr/bin/env python3
"""
Module QT4ACTION -- Python Qt 4 Action Objects
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI objects for handling user actions.
"""

from PyQt4 import Qt as qt

from plib.gui._base import action
from plib.gui.defs import *

from ._qt4common import _PQtCommunicator, _PQtWidgetBase


class PQtMenu(_PQtWidgetBase, qt.QMenuBar, action.PMenuBase):
    """Customized Qt menu class.
    """
    
    popupclass = qt.QMenu
    
    def __init__(self, mainwidget):
        qt.QMenuBar.__init__(self, mainwidget)
        action.PMenuBase.__init__(self, mainwidget)
        mainwidget.setMenuBar(self)
    
    def _add_popup(self, title, popup):
        popup.setTitle(title)
        self.addMenu(popup)
    
    def _add_popup_action(self, act, popup):
        popup.addAction(act)


class PQtToolBar(_PQtWidgetBase, qt.QToolBar, action.PToolBarBase):
    """Customized Qt toolbar class.
    """
    
    def __init__(self, mainwidget):
        qt.QToolBar.__init__(self, mainwidget)
        if mainwidget.large_icons:
            sz = self.iconSize()
            self.setIconSize(qt.QSize(
                *[i * 3 / 2 for i in (sz.width(), sz.height())]))
        if mainwidget.show_labels:
            style = qt.Qt.ToolButtonTextUnderIcon
        else:
            style = qt.Qt.ToolButtonIconOnly
        self.setToolButtonStyle(style)
        action.PToolBarBase.__init__(self, mainwidget)
        mainwidget.addToolBar(self)
    
    def add_action(self, act):
        self.addAction(act)
    
    def add_separator(self):
        self.addSeparator()


class PQtAction(_PQtCommunicator, qt.QAction, action.PActionBase):
    """Customized Qt action class.
    """
    
    def __init__(self, key, mainwidget):
        qt.QAction.__init__(self, mainwidget)
        self.setIcon(qt.QIcon(qt.QPixmap(self.get_icon_filename(key))))
        self.setText(self.get_menu_str(key))
        self.setToolTip(self.get_toolbar_str(key))
        s = self.get_accel_str(key)
        if s is not None:
            self.setShortcut(qt.QKeySequence(s))
        action.PActionBase.__init__(self, key, mainwidget)
    
    def enable(self, enabled):
        self.setEnabled(enabled)
